/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * eventhandlers-plot-draw - gtk event handlers related to drawing the
 * plot
 */

#include "eventhandlers-plot-draw.xh"
#include "paintapi-gtk.xh"

gboolean plot_configure_event(GtkWidget *widget, GdkEventConfigure *do_not_use, my_plot_view_t *view) {
   plot_size_or_font_changed(view->parent);
   return 0;
}

gboolean plot_map_event(GtkWidget *widget, GdkEvent *do_not_use, my_plot_view_t *view) {
   //printf("  --  map\n");

   view->drawable = view->da->window;

   view->draw.paintapi = paintapi_gtk_new(view->drawable, &view->draw);

   // set the background part 1 - create new style for widget
   GtkStyle *style = gtk_widget_get_style(view->da);
   style = gtk_style_copy(style);
   gtk_widget_set_style(view->da, style);

   // set the background part 2
   GdkColor bg;
   bg.red = bg.green = bg.blue = 0;
   gdk_colormap_alloc_color(gdk_colormap_get_system(), &bg, 0, 0);
   gdk_window_set_background(view->drawable, &bg);

   // set cursor
   GdkCursor *cursor = gdk_cursor_new(GDK_DRAFT_SMALL);
   gdk_window_set_cursor(view->da->window, cursor);
   gdk_cursor_unref(cursor);

   // fake configure since it doesn't occur at startup
   plot_configure_event(widget, NULL, view);

   return 0;
}

gboolean plot_expose_event(GtkWidget *widget, GdkEventExpose *event, my_plot_view_t *view) {
   if(view->conc > 0) {
      // still not optimal
      view->normal_expose++;
      //printf(" -> normal_expose++\n");
      return 0;
   }
   int mytrans = ++view->parent->dsc_x->num_changes + view->dsc_y->num_changes;
   view->conc++;
   //    print "->$mytrans view->conc\n";

   //int mvi; for(mvi=0; mvi<view->parent->view_count; ++mvi) if(view->parent->view[mvi] == view) break;
   //printf("%*sexpose %d\n", indent++*3, "", mvi);

   view->box = COORD_UNDEF;

   reset_view_markers(view);

   plot_draw(&view->draw);

   show_markers(view->parent, 1);

   if(mytrans != view->parent->dsc_x->num_changes + view->dsc_y->num_changes || view->normal_expose) {
      //printf("%d pending exposes, queue draw\n", view->normal_expose);
      //printf("%*sRedraw self (%d)\n", indent*3, "", mvi);
      gtk_widget_queue_draw(view->da);
   } else {
      for(int vi=0; vi<view->parent->view_count; ++vi) {
	 if(view->parent->view[vi]->normal_expose) {
	    //printf("%*sRedraw %d\n", indent*3, "", vi);
	    plot_view_need_redraw(view->parent->view[vi]);
	    break;
	 }
      }
   }
   view->normal_expose = 0;
   view->conc--;

   //printf("%*s/expose %d\n", --indent*3, "", mvi);
   
   return 0;
}

void screen_size_changed_event(GdkScreen *screen, my_plot_t *plot) {
   // works only with xrandr, not with ctrl-alt-+ and ctrl-alt--
   plot_size_or_font_changed(plot);
}
