/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * eventhandlers-plot-mouse - gtk event handlers related handling
 * mouse events for the plot
 */

#include <gtk/gtktogglebutton.h>

#include "eventhandlers-plot-mouse.xh"

gboolean plot_button_press_event(GtkWidget *widget, GdkEventButton *event, my_plot_view_t *view) {
   int x = event->x;
   int y = event->y;

#ifdef USE_FAM
   boolean_t tailing = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(view->parent->checkbutton_tail));
#else
   const boolean_t tailing = 0;
#endif

   switch((int)event->type) {
/*
  case GDK_2BUTTON_PRESS: {

  printf("dpress %d %d\n", x, y);
  plot_move_to(view, view->draw.plot_xe/2, view->draw.plot_yde/2);
  break;
  }
*/

      case GDK_BUTTON_PRESS:
	 // printf("press %d %d\n", x, y);
	 if(view->bmode == 0) {

	    view->bx = x;
	    view->by = y;

	    destroy_markers(view->parent);

	    if(plot_draw_check_if_hit_legend_box(&view->draw, x, y)) {
	       my_plot_line_info_t *line = plot_draw_check_if_hit_legend_item(&view->draw, x, y);

	       if(line) {
		  switch(event->button) {
		     case 1:
			view->bstate = line->enabled = !line->enabled;
			view->bmode = 4;

			plot_view_need_redraw(view);
			update_view_minmax(view);
			break;

			/*
			  default:
			  diff mode, TODO scales don't work and we need a time unit also
			  view->bstate = line->mode == LINE_MODE_NORMAL;
			  line->mode = view->bstate ? LINE_MODE_DIFF : LINE_MODE_NORMAL;
			  view->bmode = 5;

			  plot_view_need_redraw(view);
			  update_view_minmax(view);
			  break;
			*/
		  }
	       }
	       return 0;
	    }

	    // update scale pos according to press location

	    int snap_x_threshold = 40; // view->draw.plot_xe / 10;
	    int snap_y_threshold = 40; // view->draw.plot_yde / 10;
	    int snap_x = x < snap_x_threshold ? 0 : x >= view->draw.plot_xe - snap_x_threshold ? view->draw.plot_xe-1 : x;
	    int snap_y = y < snap_y_threshold ? 0 : y >= view->draw.plot_yde - snap_y_threshold ? view->draw.plot_yde-1 : y;

	    data_scrollbar_controller_set_zoompoint(view->parent->dsc_x, ((double)snap_x) / view->draw.plot_xe);
	    data_scrollbar_controller_set_zoompoint(view->dsc_y, 1 - ((double)snap_y) / view->draw.plot_yde);

	    view->bspeed = event->state & GDK_SHIFT_MASK ? 4 : 1;

	    if(event->button > 1) {
	       view->bx = x;
	       view->bmode = 2;
	       view->bxloglength = data_scrollbar_controller_get_page_length_log(view->parent->dsc_x);
	       view->byloglength = data_scrollbar_controller_get_page_length_log(view->dsc_y);

	       // when close to border, zoom to border
	       if(tailing || x >= view->draw.plot_xe - 30) {
		  data_scrollbar_controller_set_zoompoint(view->parent->dsc_x, 1.0);
	       } else if(x < 30) { // ##TODO## conf 30
		  data_scrollbar_controller_set_zoompoint(view->parent->dsc_x, 0.0);
	       }

	       if(y < 30) {
		  data_scrollbar_controller_set_zoompoint(view->dsc_y, 1.0);
	       } else if(y >= view->draw.plot_yde - 30) {
		  data_scrollbar_controller_set_zoompoint(view->dsc_y, 0.0);
	       }

	    } else if(event->state & GDK_CONTROL_MASK) {
	       view->bmode = 3;
	       view->box = COORD_UNDEF;

	    } else {
	       view->bx = tailing ? view->draw.plot_xe : x;
	       view->bmode = 1;
	       view->bxoff = data_scrollbar_controller_get_page_off(view->parent->dsc_x);
	       view->byoff = data_scrollbar_controller_get_page_off(view->dsc_y);
	    }
	 }
	 break;
   }

   return 0;
}

#define LIMIT(v,min,max) do{ if(v<(min)) v=min; else if(v>(max)) v=max; } while(0)

void plot_motion_generic(my_plot_view_t *view, int x, int y, int release) {
#ifdef USE_FAM
   boolean_t tailing = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(view->parent->checkbutton_tail));
#else
   const boolean_t tailing = 0;
#endif

   switch(view->bmode) {
      case 0:
	 if(plot_draw_check_if_hit_legend_box(&view->draw, x, y)) {
	    destroy_markers(view->parent);
	 } else {
	    set_and_show_markers(view, x, y);
	 }
	 break;

      case 1:
	 plot_move_to(view, tailing ? view->draw.plot_xe : x, y);
	 if(release) {
	    add_to_zoom_history(view->parent);
	 }
	 break;

      case 2: {
	 // ##TODO## conf 5.0 and 2.0
	 data_scrollbar_controller_set_page_length_log(view->parent->dsc_x, view->bxloglength - (x - view->bx) * view->bspeed * 5.0 / view->draw.plot_xe);
	 data_scrollbar_controller_set_page_length_log(view->dsc_y, view->byloglength - (y - view->by) * view->bspeed * 2.0 / view->draw.plot_yde);
	 if(release) {
	    add_to_zoom_history(view->parent);
	 }
	 break;
      }

      case 3: {
	 LIMIT(view->bx, 0, view->draw.plot_xe);
	 LIMIT(view->by, 0, view->draw.plot_yde);
	 LIMIT(x, 0, view->draw.plot_xe);
	 LIMIT(y, 0, view->draw.plot_yde);

	 view->draw.paintapi->gc_set_dashes(view->draw.paintapi, view->draw.cursor_marker_gc, -view->hline, cursor_marker_dashes, SZ(cursor_marker_dashes));
	 if(!release) {
	    view->draw.paintapi->draw_rectangle(view->draw.paintapi, view->draw.cursor_marker_gc, 0, view->bx, view->by, x, y);
	 }
	 if(view->box != COORD_UNDEF) {
	    view->draw.paintapi->draw_rectangle(view->draw.paintapi, view->draw.cursor_marker_gc, 0, view->bx, view->by, view->box, view->boy);
	 }
	 if(!release) {
	    view->box = x;
	    view->boy = y;
	 } else {
	    int x1 = view->bx;
	    int y1 = view->by;

	    y = view->draw.plot_yde - y;
	    y1 = view->draw.plot_yde - y1;

	    if(x < x1) { int t = x; x = x1; x1 = t; }
	    if(y < y1) { int t = y; y = y1; y1 = t; }

	    double ox = data_scrollbar_controller_get_page_off(view->parent->dsc_x);
	    double oy = data_scrollbar_controller_get_page_off(view->dsc_y);
	    double lx = data_scrollbar_controller_get_page_length(view->parent->dsc_x);
	    double ly = data_scrollbar_controller_get_page_length(view->dsc_y);

	    //printf("%d,%d %d,%d\n", x1, y1, x-x1, y-y1);

	    //printf("%lf,%lf %lf,%lf\n", ox, oy, lx, ly);

	    ox += lx * x1 / view->draw.plot_xe;
	    oy += ly * y1 / view->draw.plot_yde;

	    lx *= (x - x1) / (double)view->draw.plot_xe;
	    ly *= (y - y1) / (double)view->draw.plot_yde;

	    //printf("%lf,%lf %lf,%lf\n\n", ox, oy, lx, ly);

	    // after area zoom, always put center as zoom point
	    data_scrollbar_controller_set_zoompoint(view->parent->dsc_x, 0.5);
	    data_scrollbar_controller_set_zoompoint(view->dsc_y, 0.5);

	    // update page lengths
	    data_scrollbar_controller_set_page_length(view->parent->dsc_x, lx);
	    data_scrollbar_controller_set_page_length(view->dsc_y, ly);

	    // now, in case either page length subceeded the minimum page length, we adjust offset so that the selected zoom area becomes centered in view
	    double nlx = data_scrollbar_controller_get_page_length(view->parent->dsc_x);
	    double nly = data_scrollbar_controller_get_page_length(view->dsc_y);

	    ox -= (nlx - lx) / 2;
	    oy -= (nly - ly) / 2;

	    // update offsets
	    data_scrollbar_controller_set_page_off(view->parent->dsc_x, ox);
	    data_scrollbar_controller_set_page_off(view->dsc_y, oy);

	    //printf("%lf,%lf %lf,%lf\n\n", data_scrollbar_controller_get_page_off(view->parent->dsc_x), data_scrollbar_controller_get_page_off(view->dsc_y), nlx, nly);

	    view->box = COORD_UNDEF;

	    add_to_zoom_history(view->parent);
	 }
	 break;
      }

      case 4: {
	 if(!view->draw.show_disabled) break;
	 if(!plot_draw_check_if_hit_legend_box(&view->draw, x, y)) break;

	 my_plot_line_info_t *line = plot_draw_check_if_hit_legend_item(&view->draw, x, y);

	 if(line) {
	    int changed = line->enabled != view->bstate;
	    if(changed) {
	       line->enabled = view->bstate;

	       plot_view_need_redraw(view);
	       update_view_minmax(view);
	    }
	 }
	 break;
      }

      case 5: {
	 if(!plot_draw_check_if_hit_legend_box(&view->draw, x, y)) break;

	 my_plot_line_info_t *line = plot_draw_check_if_hit_legend_item(&view->draw, x, y);

	 if(line) {
	    int changed = line->mode != (view->bstate ? LINE_MODE_DIFF : LINE_MODE_NORMAL);
	    if(changed) {
	       line->mode = view->bstate ? LINE_MODE_DIFF : LINE_MODE_NORMAL;

	       plot_view_need_redraw(view);
	       update_view_minmax(view);
	    }
	 }
	 break;
      }

      default:
	 fprintf(stderr, "ERROR: bmode = %d\n", view->bmode);
	 break;
   }
}

gboolean plot_motion_notify_event(GtkWidget *wid, GdkEventMotion *event, my_plot_view_t *view) {
   int x = event->x;
   int y = event->y;

   plot_motion_generic(view, x, y, 0);

   return 0;
}

gboolean plot_button_release_event(GtkWidget *widget, GdkEventButton *event, my_plot_view_t *view) {
   int x = event->x;
   int y = event->y;

   plot_motion_generic(view, x, y, 1);

   view->bmode = 0;

   return 0;
}

gboolean plot_leave_notify_event(GtkWidget *widget, GdkEventCrossing *event, my_plot_view_t *view) {
   destroy_markers(view->parent);

   return 0;
}
