/*
 * Plot - plot time-based data on screen and to file with interactive controls
 * Copyright (C) 2006  Jonas Berlin <xkr47@outerspace.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****
 * sample_density_dialog - manage the sample density adjustment dialog
 */

#include <gtk/gtkdialog.h>
#include <gtk/gtkrange.h>
#include <gtk/gtkscale.h>

#include <math.h>

#include "sample_density_dialog.xh"

typedef struct {
      GtkRange *range_sample_density;
      GtkLabel *label_render_time;
      my_plot_t *plot;
} my_plot_sample_density_callback_data_t;

static void sample_density_response(GtkDialog *dialog, gint response, my_plot_sample_density_callback_data_t *sdcbd) {
   if(response == GTK_RESPONSE_OK || response == GTK_RESPONSE_APPLY) {
      double total_render_time = 0;
      double sample_density = exp10(gtk_range_get_value(sdcbd->range_sample_density));
      for(int vi=0; vi<sdcbd->plot->view_count; ++vi) {
	 sdcbd->plot->view[vi]->draw.samples_per_pixel = sample_density;
	 plot_view_need_redraw(sdcbd->plot->view[vi]);
	 gdk_window_process_updates(sdcbd->plot->view[vi]->drawable, 0);

	 total_render_time += sdcbd->plot->view[vi]->draw.last_render_time;
      }
      char tmp[30];
      sprintf(tmp, "<b>%.3lf</b> seconds", total_render_time);
      gtk_label_set_markup(sdcbd->label_render_time, tmp);
   }

   if(response != GTK_RESPONSE_APPLY) {
      bzero(sdcbd, sizeof(my_plot_sample_density_callback_data_t));
      free(sdcbd);
      gtk_widget_destroy(GTK_WIDGET(dialog));
   }
}

static gchar* sample_density_format(GtkScale *scale, gdouble newval, gpointer user_data) {
   if(newval < -0.01) {
      return g_strdup_printf("%.1lf pixels per sample", exp10(-newval));
   } else if(newval <= 0.01) {
      return g_strdup("1 sample per pixel");
   } else {
      return g_strdup_printf("%.1lf samples per pixel", exp10(newval));
   }
}

void sample_density_dialog_run(my_plot_t *plot) {
   my_plot_sample_density_callback_data_t *sdcbd = zmalloc_c(sizeof(my_plot_sample_density_callback_data_t));
   sdcbd->plot = plot;

   GladeXML *gladexml = glade_xml_new(plot->gladefile, "dialog_sample_density", NULL);
   GtkWidget *dialog = glade_xml_get_widget(gladexml, "dialog_sample_density");

   sdcbd->range_sample_density = GTK_RANGE(glade_xml_get_widget(gladexml, "hscale_sample_density"));
   gtk_range_set_value(sdcbd->range_sample_density, log10(plot->view[0]->draw.samples_per_pixel));

   sdcbd->label_render_time = GTK_LABEL(glade_xml_get_widget(gladexml, "label_render_time"));
   double total_render_time = 0;
   for(int vi=0; vi<sdcbd->plot->view_count; ++vi) {
      total_render_time += sdcbd->plot->view[vi]->draw.last_render_time;
   }
   char tmp[30];
   sprintf(tmp, "<b>%.3lf</b> seconds", total_render_time);
   gtk_label_set_markup(sdcbd->label_render_time, tmp);

   g_signal_connect(sdcbd->range_sample_density, "format-value", G_CALLBACK(sample_density_format), sdcbd);
   g_signal_connect(dialog, "response", G_CALLBACK(sample_density_response), sdcbd);

   gtk_widget_show(GTK_WIDGET(dialog));
}
